#define F_CPU 16000000
#include "lcd.h"
#include <util/delay.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <avr/pgmspace.h>
#include "ip_arp_udp_tcp.h"
#include "enc28j60.h"
#include "timeout.h"
#include "avr_compat.h"
#include "net.h"


/********************************************************************************
 *																				*
 *							LCD Display Related Functions						*
 *																				*
 ********************************************************************************/


void LCD_init()
{
	LcdDATA_DDR |=  (1<<LcdD4)|(1<<LcdD5)|(1<<LcdD6)|(1<<LcdD7);	// setam pinii de date ca pini de iesire
	LcdCMD_DDR  |=  (1<<LcdRS)|(1<<LcdRW)|(1<<LcdE);				// setam pinii de comenzi ca pini de iesire

	LCD_waitNotBusy();
	
	LcdCMD_PORT   &= ~(1<<LcdRS);									// Setam linia RS pe low
	LcdCMD_PORT   &= ~(1<<LcdRW);									// Setam linia RW pe low (acum suntem in modul de trimis instructiuni)
	LcdDATA_PORT  &= ~(1<<LcdD4)&~(1<<LcdD6)&~(1<<LcdD7); 		// Specificam ca vrem 4 fire de date, prima comanda (LcdD5 activ, restul nu)
	LcdDATA_PORT  |=  (1<<LcdD5);									// setam pinii de comenzi ca pini de iesire
	
	LcdCMD_PORT |=  (1<<LcdE);					// Setam linia E(nable) pe high; aceasta ii specifica LCD-ului sa preia datele
	LCD_waitInstructions(12);					// Asteptam o perioada de timp T
	LcdCMD_PORT &= ~(1<<LcdE );				// Setam linia E(nable) pe low; transferul s-a terminat

	LCD_writeInstruction(LCD_INSTR_4wire); 		// Incarcam comanda: 4 bit data, 2 lines, 5x8 font
	LCD_writeInstruction(LCD_INSTR_display); 	// Display On, Cursor On, Blinking On
	LCD_writeInstruction(0x06);					// Increment, no shift
	LCD_writeInstruction(0x01);					// Clear Display

	
}

void LCD_writeInstruction(unsigned char _instruction)
{			
	
	LCD_waitNotBusy(); 							// asteptam ca LCD-ul sa fie liber sa primeasca comenzile	
	RS_LOW();									// setam pinul RS pe low (low=instructiuni, high=date)
	RW_LOW();									// setam pinul RW pe low (suntem in modul de comenzi acum)
	LCD_write(_instruction);					// apelam procedura ce trimite byte-ul pe firele de date
}


void LCD_writeData(unsigned char _data)
{
	LCD_waitNotBusy(); 							// asteptam ca LCD-ul sa fie liber sa primeasca comenzile	
	RS_HIGH();									// setam pinul RS pe high (low=instructiuni, high=date)
	RW_LOW();									// setam pinul RW pe low (suntem in modul de date acum)
	LCD_write(_data);							// apelam procedura ce trimite byte-ul pe firele de date

}

void LCD_write(unsigned char _byte)
{
	unsigned char _byte2;
	
	_byte2 = _byte>>4;

	LcdDATA_PORT &= ~(1<<LcdD4);
	if ( bit_is_set( _byte2, 0 ) )
		LcdDATA_PORT |= (1<<LcdD4);
		
	LcdDATA_PORT &= ~(1<<LcdD5);
	if ( bit_is_set( _byte2, 1 ) )
		LcdDATA_PORT |= (1<<LcdD5);

	LcdDATA_PORT &= ~(1<<LcdD6);
	if ( bit_is_set( _byte2, 2 ) )
		LcdDATA_PORT |= (1<<LcdD6);

	LcdDATA_PORT &= ~(1<<LcdD7);
	if ( bit_is_set( _byte2, 3 ) )
		LcdDATA_PORT |= (1<<LcdD7);

	LcdCMD_PORT |= (1<<LcdE);						// Setam Pinul E pe high
	LCD_waitInstructions(12);						// Asteptam o perioada de timp T
	LcdCMD_PORT &= ~(1<<LcdE);						// Setam Pinul E pe low

	LCD_waitInstructions(12);						// Asteptam o perioada de timp T
	
	LcdDATA_PORT &= ~(1<<LcdD4);
	if ( bit_is_set( _byte, 0 ) )
		LcdDATA_PORT |= (1<<LcdD4);
	
	LcdDATA_PORT &= ~(1<<LcdD5);
	if ( bit_is_set( _byte, 1 ) )
		LcdDATA_PORT |= (1<<LcdD5);

	LcdDATA_PORT &= ~(1<<LcdD6);
	if ( bit_is_set( _byte, 2 ) )
		LcdDATA_PORT |= (1<<LcdD6);

	LcdDATA_PORT &= ~(1<<LcdD7);
	if ( bit_is_set( _byte, 3 ) )
		LcdDATA_PORT |= (1<<LcdD7);

	LcdCMD_PORT |= (1<<LcdE);						// Setam Pinul E pe high
	LCD_waitInstructions(12);						// Asteptam o perioada de timp T
	LcdCMD_PORT &= ~(1<<LcdE);						// Setam Pinul E pe low
	LCD_waitInstructions(12);
}

void LCD_waitNotBusy()
{
_delay_ms(10);
/*
	unsigned char _loop = 1;

	while (_loop)
	{
		LcdDATA_DDR &= ~(1<<LcdD4 | 1<<LcdD5 | 1<<LcdD6 | 1<<LcdD7);	// Setam pinii de date de la LCD pe in pt a citi busy flag
		LcdDATA_PORT &= ~(1<<LcdD4 | 1<<LcdD5 | 1<<LcdD6 | 1<<LcdD7); 	// Dezactivam pullup resistor pentru pinii de in
		
		LcdCMD_PORT &= ~(1<<LcdE);					// Setam pin-ul e pe low; ar trebui sa fie deja low, doar ne asiguram
		LcdCMD_PORT &= ~(1<<LcdRS);					// Setam pinul RS pe low
		LcdCMD_PORT |=  (1<<LcdRW);					// Setam pinul RW pe high (acum suntem in modul de interogare busy/adr)
		
		LcdCMD_PORT |= (1<<LcdE);						// Setam Pinul E pe high
		LCD_waitInstructions(12);						// Asteptam o perioada de timp T
		_loop = LcdDATA_PIN & (1<<LcdD7);				// Citim busy flag-ul
		LcdCMD_PORT &= ~(1<<LcdE);						// Setam Pinul E pe low
		
		LCD_waitInstructions(12);						// Asteptam o perioada de timp T
		
		LcdCMD_PORT |= (1<<LcdE);						// Setam Pinul E pe high
		LCD_waitInstructions(12);						// Asteptam o perioada de timp T
		LcdCMD_PORT &= ~(1<<LcdE);						// Setam Pinul E pe low

		LcdDATA_DDR |= (1<<LcdD4 | 1<<LcdD5 | 1<<LcdD6 | 1<<LcdD7); // Setam Portul de LCD ca port de iesire la loc
	}*/
}

void LCD_waitInstructions(unsigned char _instructions)
{
	while (_instructions--)
		;
}


void LCD_printDecimal2u(unsigned int _n)
{
	unsigned char tmp=0;
	
	// Extragem sutele
	while(_n>=100)
		_n-=100;

	while(_n>=10){
		tmp++;
		_n-=10;
	}

	LCD_writeData(tmp+'0');
	LCD_writeData(_n+'0');
}

void LCD_printHexa(unsigned int _n)
{
	unsigned char _tmp = _n>>4;
	if (_tmp>9)
		_tmp += 'A'-10;
	else
		_tmp += '0';
	LCD_writeData( _tmp );
	_tmp = _n & 0x0F;
	if (_tmp>9)
		_tmp += 'A'-10;
	else
		_tmp += '0';
	LCD_writeData( _tmp );
}


void LCD_print(char* _msg)
{
	unsigned char i=0;
	for( ; _msg[i]!=0 && i<16; i++)
		LCD_writeData( _msg[i] );
}

void LCD_print2(char* _msg1, char* _msg2)
{
	LCD_writeInstruction(LCD_INSTR_clearDisplay);
	LCD_print(_msg1);
	LCD_writeInstruction(LCD_INSTR_nextLine);
	LCD_print(_msg2);
}


/********************************************************************************
 *																				*
 *							Ethernet Related Functions							*
 *																				*
 ********************************************************************************/

// Please modify the following two lines.
// Attention! MAC and IP have to be unique in your local area network.
// You can not have the same numbers in two devices. If a conflict appears
// change it!
static uint8_t mymac[6]     =   {0x54, 0x55, 0x58, 0x10, 0x00, 0x24};
static uint8_t myip[4]      =   {10, 38, 243, 240};
#define MY_IP_STR               "10.38.243.240"

// Your IP.
uint8_t yip[4];

// Base url (you can put a DNS name instead of an IP address if you have
// a DNS server (baseurl must end in "/").
//static char baseurl[]       =   "http://10.38.243.240/";

// Listen port for tcp/www (max range 1-254).
static uint16_t mywwwport   =   80;

#define BUFFER_SIZE             400

unsigned char send_buf[BUFFER_SIZE + 1];
unsigned char recv_buf[BUFFER_SIZE + 1];



// creeaza un header ip setand atat sursa cat si destinatia ca ip-ul dispozitivului
void make_ip_to_self(uint8_t *buf)
{
	// trebuie completate informatii suplimentare in antetul ip
	uint8_t i=0;
	while(i<4){
			buf[IP_DST_P+i]=myip[i];
			buf[IP_SRC_P+i]=myip[i];
			i++;
	}
	fill_ip_hdr_checksum(buf);
}

// creeaza header de ethernet, setand atat sursa si destinatia ca mac-ul dispozitivului
void make_eth_to_self(uint8_t *buf)
{
        uint8_t i=0;
        //
        //copy the destination mac from the source and fill my mac into src
        while(i<6){
                buf[ETH_DST_MAC +i]=mymac[i];
                buf[ETH_SRC_MAC +i]=mymac[i];
                i++;
        }
}

// formeaza pachetul de test si il trimite
// More work required 
// TODO
void send_local_echo_reply_from_scratch(uint8_t *buf,uint16_t len)
{
        make_eth_to_self(buf);
        make_ip_to_self(buf);
        buf[ICMP_TYPE_P]=ICMP_TYPE_ECHOREPLY_V;
        // we changed only the icmp.type field from request(=8) to reply(=0).
        // we can therefore easily correct the checksum:
        if (buf[ICMP_CHECKSUM_P] > (0xff-0x08)){
                buf[ICMP_CHECKSUM_P+1]++;
        }
        buf[ICMP_CHECKSUM_P]+=0x08;
        enc28j60PacketSend(len,buf);
}

int main()
{
	int sz,i,cnt;
	int pos_found;
	char send_msg[200],recv_msg[200];
	char send_buff[200],recv_buff[200];
	
	
	LCD_init();
	LCD_print2("*** READY? ***"," INTRODUCETI MUFELE ");
	delay_ms(10000);
	
	LCD_print2("WE REALLY HOPE","YOU ARE FAST");
	
	//initTimer2();
	sei();

	DDRD |= (1<<DDD4);
	delay_ms(10);
	
	PORTD &= ~(1<<PD4);
    delay_ms(10);

	// Set output to Vcc, reset inactive.
	PORTD |= (1<<PD4);
	delay_ms(200);

	// Initialize enc28j60.
	enc28j60Init(mymac);
	delay_ms(20);
	
	enc28j60PhyWrite(PHLCON, 0x476);
    delay_ms(20);
	
	init_ip_arp_udp_tcp(mymac, myip, mywwwport);
	
	LCD_print2("** Phase 1 **","END INIT");
	delay_ms(3000);
	
	
	for(i=0;i<100;i++)
		send_buff[i] = 'a';
	send_buff[100] = 0;
	cnt = 0;
	snprintf(send_msg,25,"%d-%s",cnt++,send_buff);

	enc28j60PacketSend(100, (unsigned char*)send_buff);
	delay_ms(3000);
	
	LCD_print2(send_msg,"waiting recv");
	delay_ms(1000);
	
	sz = enc28j60PacketReceive(200,(unsigned char*)recv_buff);
	snprintf(recv_msg,25,"%d-%s",sz,recv_buff);
	
	LCD_print2(send_msg,recv_msg);
	delay_ms(3000);
	
	// ciclu de repetare a trimiterii de pachete
	pos_found = -1;
	while (1) {
		enc28j60PacketSend(100, (unsigned char*)send_buff);
		delay_ms(8000);
		snprintf(send_msg,100,"%d-%s",cnt++,send_buff);
		sz = enc28j60PacketReceive(200,(unsigned char*)recv_buff);
		
		pos_found = -1;
		for(i=0;i<sz;i++)
			if(recv_buff[i]=='a'){
				pos_found = i;
				break;
			}
		snprintf(recv_msg,25,"%d,%d-%s",pos_found,sz,(unsigned char*)recv_buff);
		LCD_print2(send_msg,recv_msg);
		delay_ms(1000);
	}
	
	return 0;
}